@@ -2,6 +2,7 @@
 Wrapper for safely working with curses subwindows.
 """
 
+import sys
 import traceback
 import curses
 from threading import RLock
@@ -55,8 +56,9 @@ class Panel():
     # implementations aren't entirely deterministic (for instance panels
     # might chose their height based on its parent's current width).
     
-    self.parent = parent
     self.panelName = name
+    self.parent = parent
+    self.visible = True
     self.top = top
     self.height = height
     self.width = width
@@ -98,6 +100,23 @@ class Panel():
       self.parent = parent
       self.win = None
   
+  def isVisible(self):
+    """
+    Provides if the panel's configured to be visible or not.
+    """
+    
+    return self.visible
+  
+  def setVisible(self, isVisible):
+    """
+    Toggles if the panel is visible or not.
+    
+    Arguments:
+      isVisible - panel is redrawn when requested if true, skipped otherwise
+    """
+    
+    self.visible = isVisible
+  
   def getTop(self):
     """
     Provides the position subwindows are placed at within its parent.
@@ -169,7 +188,7 @@ class Panel():
     if setWidth != -1: newWidth = min(newWidth, setWidth)
     return (newHeight, newWidth)
   
-  def draw(self, subwindow, width, height):
+  def draw(self, width, height):
     """
     Draws display's content. This is meant to be overwritten by 
     implementations and not called directly (use redraw() instead). The
@@ -177,10 +196,8 @@ class Panel():
     a column less than the actual space.
     
     Arguments:
-      sudwindow - panel's current subwindow instance, providing raw access to
-                  its curses functions
-      width     - horizontal space available for content
-      height    - vertical space available for content
+      width  - horizontal space available for content
+      height - vertical space available for content
     """
     
     pass
@@ -197,6 +214,9 @@ class Panel():
                     abandoned
     """
     
+    # skipped if not currently visible
+    if not self.isVisible(): return
+    
     # if the panel's completely outside its parent then this is a no-op
     newHeight, newWidth = self.getPreferredSize()
     if newHeight == 0:
@@ -221,11 +241,70 @@ class Panel():
     try:
       if forceRedraw:
         self.win.erase() # clears any old contents
-        self.draw(self.win, self.maxX - 1, self.maxY)
+        self.draw(self.maxX - 1, self.maxY)
       self.win.refresh()
     finally:
       CURSES_LOCK.release()
   
+  def hline(self, y, x, length, attr=curses.A_NORMAL):
+    """
+    Draws a horizontal line. This should only be called from the context of a
+    panel's draw method.
+    
+    Arguments:
+      y      - vertical location
+      x      - horizontal location
+      length - length the line spans
+      attr   - text attributes
+    """
+    
+    if self.win and self.maxX > x and self.maxY > y:
+      try:
+        drawLength = min(length, self.maxX - x)
+        self.win.hline(y, x, curses.ACS_HLINE | attr, drawLength)
+      except:
+        # in edge cases drawing could cause a _curses.error
+        pass
+  
+  def vline(self, y, x, length, attr=curses.A_NORMAL):
+    """
+    Draws a vertical line. This should only be called from the context of a
+    panel's draw method.
+    
+    Arguments:
+      y      - vertical location
+      x      - horizontal location
+      length - length the line spans
+      attr   - text attributes
+    """
+    
+    if self.win and self.maxX > x and self.maxY > y:
+      try:
+        drawLength = min(length, self.maxY - y)
+        self.win.vline(y, x, curses.ACS_VLINE | attr, drawLength)
+      except:
+        # in edge cases drawing could cause a _curses.error
+        pass
+  
+  def addch(self, y, x, char, attr=curses.A_NORMAL):
+    """
+    Draws a single character. This should only be called from the context of a
+    panel's draw method.
+    
+    Arguments:
+      y    - vertical location
+      x    - horizontal location
+      char - character to be drawn
+      attr - text attributes
+    """
+    
+    if self.win and self.maxX > x and self.maxY > y:
+      try:
+        self.win.addch(y, x, char, attr)
+      except:
+        # in edge cases drawing could cause a _curses.error
+        pass
+  
   def addstr(self, y, x, msg, attr=curses.A_NORMAL):
     """
     Writes string to subwindow if able. This takes into account screen bounds
@@ -358,8 +437,16 @@ class Panel():
     
     # Displays the text field, blocking until the user's done. This closes the
     # text panel and returns userInput to the initial text if the user presses
-    # escape.
-    textbox = curses.textpad.Textbox(inputSubwindow, True)
+    # escape. Insert mode is available in Python 2.6+, before that the
+    # constructor only accepted a subwindow argument as per:
+    # https://trac.torproject.org/projects/tor/ticket/2354
+    
+    majorVersion, minorVersion = sys.version_info[:2]
+    if majorVersion == 2 and minorVersion >= 6:
+      textbox = curses.textpad.Textbox(inputSubwindow, True)
+    else:
+      textbox = curses.textpad.Textbox(inputSubwindow)
+    
     userInput = textbox.edit(lambda key: _textboxValidate(textbox, key)).strip()
     if textbox.lastcmd == curses.ascii.BEL: userInput = None
     
